(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/backup.js                                                    //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var ChildProcess = Npm.require('child_process');                       // 17
var Future = Npm.require('fibers/future');                             // 18
var Promise = Npm.require('es6-promise').Promise;                      // 19
var Capnp = Npm.require('capnp');                                      // 20
                                                                       //
var GrainInfo = Capnp.importSystem('sandstorm/grain.capnp').GrainInfo;
                                                                       //
var TOKEN_CLEANUP_MINUTES = 15;                                        // 24
var TOKEN_CLEANUP_TIMER = TOKEN_CLEANUP_MINUTES * 60 * 1000;           // 25
                                                                       //
function cleanupToken(tokenId) {                                       // 27
  check(tokenId, String);                                              // 28
  waitPromise(globalBackend.cap().deleteBackup(tokenId));              // 29
  FileTokens.remove({ _id: tokenId });                                 // 30
}                                                                      //
                                                                       //
Meteor.startup(function () {                                           // 33
  // Cleanup tokens every TOKEN_CLEANUP_MINUTES                        //
  SandstormDb.periodicCleanup(TOKEN_CLEANUP_TIMER, function () {       // 35
    var queryDate = new Date(Date.now() - TOKEN_CLEANUP_TIMER);        // 36
                                                                       //
    FileTokens.find({ timestamp: { $lt: queryDate } }).forEach(function (token) {
      cleanupToken(token._id);                                         // 39
    });                                                                //
  });                                                                  //
});                                                                    //
                                                                       //
Meteor.methods({                                                       // 44
  backupGrain: function (grainId) {                                    // 45
    check(grainId, String);                                            // 46
    var grain = Grains.findOne(grainId);                               // 47
    if (!grain || !this.userId || grain.userId !== this.userId) {      // 48
      throw new Meteor.Error(403, 'Unauthorized', 'User is not the owner of this grain');
    }                                                                  //
                                                                       //
    this.unblock();                                                    // 52
                                                                       //
    var token = {                                                      // 54
      _id: Random.id(),                                                // 55
      timestamp: new Date(),                                           // 56
      name: grain.title                                                // 57
    };                                                                 //
                                                                       //
    // TODO(soon): does the grain need to be offline?                  //
                                                                       //
    var grainInfo = _.pick(grain, 'appId', 'appVersion', 'title');     // 62
                                                                       //
    FileTokens.insert(token);                                          // 64
    waitPromise(globalBackend.cap().backupGrain(token._id, this.userId, grainId, grainInfo));
                                                                       //
    return token._id;                                                  // 67
  },                                                                   //
                                                                       //
  restoreGrain: function (tokenId, identityId) {                       // 70
    check(tokenId, String);                                            // 71
    check(identityId, String);                                         // 72
    var token = FileTokens.findOne(tokenId);                           // 73
    if (!token || !isSignedUpOrDemo()) {                               // 74
      throw new Meteor.Error(403, 'Unauthorized', 'Token was not found, or user cannot create grains');
    }                                                                  //
                                                                       //
    if (isUserOverQuota(Meteor.user())) {                              // 79
      throw new Meteor.Error(402, 'You are out of storage space. Please delete some things and try again.');
    }                                                                  //
                                                                       //
    this.unblock();                                                    // 84
                                                                       //
    var grainId = Random.id(22);                                       // 86
                                                                       //
    try {                                                              // 88
      var grainInfo = waitPromise(globalBackend.cap().restoreGrain(tokenId, this.userId, grainId)['catch'](function (err) {
        console.error('Unzip failure:', err.message);                  // 91
        throw new Meteor.Error(500, 'Invalid backup file.');           // 92
      })).info;                                                        //
      if (!grainInfo.appId) {                                          // 94
        globalBackend.deleteGrain(grainId, this.userId);               // 95
        throw new Meteor.Error(500, 'Metadata object for uploaded grain has no AppId');
      }                                                                //
                                                                       //
      var action = UserActions.findOne({ appId: grainInfo.appId, userId: this.userId });
                                                                       //
      // Create variables we'll use for later Mongo query.             //
      var packageId = undefined;                                       // 102
      var appVersion = undefined;                                      // 103
                                                                       //
      // DevPackages are system-wide, so we do not check the user ID.  //
      var devPackage = DevPackages.findOne({ appId: grainInfo.appId });
      if (devPackage) {                                                // 107
        // If the dev app package exists, it should override the user action.
        packageId = devPackage.packageId;                              // 109
        appVersion = devPackage.manifest.appVersion;                   // 110
      } else if (action) {                                             //
        // The app is installed, so we can continue restoring this     //
        // grain.                                                      //
        packageId = action.packageId;                                  // 114
        appVersion = action.appVersion;                                // 115
      } else {                                                         //
        // If the package isn't installed at all, bail out.            //
        globalBackend.deleteGrain(grainId, this.userId);               // 118
        throw new Meteor.Error(500, 'App id for uploaded grain not installed', 'App Id: ' + grainInfo.appId);
      }                                                                //
                                                                       //
      if (appVersion < grainInfo.appVersion) {                         // 124
        globalBackend.deleteGrain(grainId, this.userId);               // 125
        throw new Meteor.Error(500, 'App version for uploaded grain is newer than any ' + 'installed version. You need to upgrade your app first', 'New version: ' + grainInfo.appVersion + ', Old version: ' + appVersion);
      }                                                                //
                                                                       //
      Grains.insert({                                                  // 133
        _id: grainId,                                                  // 134
        packageId: packageId,                                          // 135
        appId: grainInfo.appId,                                        // 136
        appVersion: appVersion,                                        // 137
        userId: this.userId,                                           // 138
        identityId: identityId,                                        // 139
        title: grainInfo.title,                                        // 140
        'private': true                                                // 141
      });                                                              //
    } finally {                                                        //
      cleanupToken(tokenId);                                           // 144
    }                                                                  //
                                                                       //
    return grainId;                                                    // 147
  }                                                                    //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 151
  this.route('downloadBackup', {                                       // 152
    where: 'server',                                                   // 153
    path: '/downloadBackup/:tokenId',                                  // 154
    action: function () {                                              // 155
      var token = FileTokens.findOne(this.params.tokenId);             // 156
      var response = this.response;                                    // 157
      if (!token) {                                                    // 158
        response.writeHead(404, { 'Content-Type': 'text/plain' });     // 159
        return response.end('File does not exist');                    // 160
      }                                                                //
                                                                       //
      var started = false;                                             // 163
      var filename = (token.name.replace(/["\n]/g, '') || 'backup') + '.zip';
      var sawEnd = false;                                              // 165
                                                                       //
      var stream = {                                                   // 167
        expectSize: function (size) {                                  // 168
          if (!started) {                                              // 169
            started = true;                                            // 170
            response.writeHead(200, {                                  // 171
              'Content-Length': size,                                  // 172
              'Content-Type': 'application/zip',                       // 173
              'Content-Disposition': 'attachment;filename=\"' + filename + '\"'
            });                                                        //
          }                                                            //
        },                                                             //
                                                                       //
        write: function (data) {                                       // 179
          if (!started) {                                              // 180
            started = true;                                            // 181
            response.writeHead(200, {                                  // 182
              'Content-Type': 'application/zip',                       // 183
              'Content-Disposition': 'attachment;filename=\"' + filename + '\"'
            });                                                        //
          }                                                            //
                                                                       //
          response.write(data);                                        // 188
        },                                                             //
                                                                       //
        done: function (data) {                                        // 191
          if (!started) {                                              // 192
            started = true;                                            // 193
            response.writeHead(200, {                                  // 194
              'Content-Length': 0,                                     // 195
              'Content-Type': 'application/zip',                       // 196
              'Content-Disposition': 'attachment;filename=\"' + filename + '\"'
            });                                                        //
          }                                                            //
                                                                       //
          sawEnd = true;                                               // 201
          response.end();                                              // 202
        }                                                              //
      };                                                               //
                                                                       //
      waitPromise(globalBackend.cap().downloadBackup(this.params.tokenId, stream));
                                                                       //
      if (!sawEnd) {                                                   // 208
        console.error('backend failed to call done() when downloading backup');
        if (!started) {                                                // 210
          throw new Meteor.Error(500, 'backend failed to produce data');
        }                                                              //
                                                                       //
        response.end();                                                // 214
      }                                                                //
                                                                       //
      cleanupToken(this.params.tokenId);                               // 217
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route('uploadBackup', {                                         // 221
    where: 'server',                                                   // 222
    path: '/uploadBackup',                                             // 223
    action: function () {                                              // 224
      var _this = this;                                                //
                                                                       //
      if (this.request.method === 'POST') {                            // 225
        (function () {                                                 //
          var request = _this.request;                                 // 226
          try {                                                        // 227
            (function () {                                             //
              var token = {                                            // 228
                _id: Random.id(),                                      // 229
                timestamp: new Date()                                  // 230
              };                                                       //
              var stream = globalBackend.cap().uploadBackup(token._id).stream;
                                                                       //
              FileTokens.insert(token);                                // 234
                                                                       //
              waitPromise(new Promise(function (resolve, reject) {     // 236
                request.on('data', function (data) {                   // 237
                  stream.write(data);                                  // 238
                });                                                    //
                request.on('end', function () {                        // 240
                  resolve(stream.done());                              // 241
                });                                                    //
                request.on('error', function (err) {                   // 243
                  stream.close();                                      // 244
                });                                                    //
              }));                                                     //
                                                                       //
              _this.response.writeHead(200, {                          // 248
                'Content-Length': token._id.length,                    // 249
                'Content-Type': 'text/plain'                           // 250
              });                                                      //
              _this.response.write(token._id);                         // 252
              _this.response.end();                                    // 253
            })();                                                      //
          } catch (error) {                                            //
            console.error(error.stack);                                // 255
            _this.response.writeHead(500, {                            // 256
              'Content-Type': 'text/plain'                             // 257
            });                                                        //
            _this.response.write(error.stack);                         // 259
            _this.response.end();                                      // 260
          }                                                            //
        })();                                                          //
      } else {                                                         //
        this.response.writeHead(405, {                                 // 263
          'Content-Type': 'text/plain'                                 // 264
        });                                                            //
        this.response.write('You can only POST here.');                // 266
        this.response.end();                                           // 267
      }                                                                //
    }                                                                  //
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=backup.js.map
